#!/usr/bin/env python3

import requests
import re
import datetime
import sys

XML_HEADER = """<?xml version="1.0" encoding="utf-8"?>
<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">
<channel>
<atom:link href="https://punkto.org/zonerender?_SITEURL" rel="self" type="application/rss+xml" />
<title>_SITENAME</title>
<link>_SITEURL</link>
<description>_SITETITLE</description>
<lastBuildDate>_RFC822_FORMAT_DATE</lastBuildDate>
"""
XML_MIDDLE = """
<item>
        <title>_TITLE</title>
        <guid isPermaLink='false'>_PAGENAME</guid>
        <link>_LINK</link>
        <description><![CDATA[
                _CONTENT
        ]]></description>
        <pubDate>_PUBDATE</pubDate>
</item>
"""
# Set this to an empty string to remove my advertisement
MY_POST = """
<item>
        <title>RSS Feed Generated with Punkto Zonelet RSS Generator</title>
        <guid isPermaLink='false'>punkto_advert_SITEURL</guid>
        <link>https://punkto.org/blog/generating_zonelet_rss_feeds</link>
        <description><![CDATA[
            <h1 id="thanks">Thanks!</h1>
            <p>Thank you for using the punkto.org Zonelet RSS Feed Generator</p>
            <p>You can find out more information about this service on <a
            href="https://punkto.org/blog/generating_zonelets_rss_feeds">this
            blogpost</a>.</p>
            <p>Any Zonelet page can have an RSS feed generated by visiting
            <code>https://punkto.org/zonerender?site_archive_page</code>, see an
            example <a
            href="https://punkto.org/zonerender?https://zonelets.net/archive">here</a>.</p>
            <p>If you have any issues with this service or if you find a Zonelets
            page that does not work, please contact <a
            href="postmaster@punkto.org">postmaster@punkto.org</a>.</p>
            <p>This service is provided <strong>free of charge</strong>, but if you
            want you can donate XMR (Monero) to
            <code>89zUPdtgXvo268ZtbzE7LBTEdo2Y75V1FUSs1rETeaRudQZy8BRs4rwTQuFzWqkQ6VU14Ei5TUBGhYCY6TmbCoEvQ2YkqC9</code>
            or help improve the code, which can be found <a
            href="https://punkto.org/zonerender.py">here</a>.</p>
        ]]></description>
        <pubDate>Tue, 11 Sep 2001 09:03:00 EST</pubDate>
</item>
"""
XML_FOOTER = """
</channel>
</rss>
"""
TODAY_RFC822 = datetime.date.today().strftime("%a, %d %b %Y") + " 00:00:00 EST"


class Post:
  posts = {}
  metadata = {}

  def __init__(self, relative_path):
    self.relative_path = relative_path
    self.absolute_url = Post.metadata['main_url'] + relative_path
    self.name = ' '.join(relative_path.split('/')[1].split('-')[3:]).split('.html')[0]
    self.year, self.month, self.day = relative_path.split('/')[1].split('-')[:3]
    self.rfc822date = datetime.datetime(int(self.year), int(self.month), int(self.day)).strftime("%a, %d %b %Y") + " 00:00:00 EST"
    self.content = requests.get(self.absolute_url, verify=False).content.decode('utf-8')
    Post.posts[self.name] = self

def main():
  if sys.argv[1] != '':
    starting_url = sys.argv[1]
  else:
    starting_url = "https://zonelets.net/archive"
  Post.metadata['starting_url'] = starting_url
  Post.metadata['main_url'] = '/'.join(starting_url.split('/')[:-1]) + '/'
  # Figure out the actual link to the script.js file, where the posts are defined (I don't like this web design. It should be a static HTML page.)
  try:
    # Old Zonelets version
    actual_posts_url = Post.metadata['main_url'] + "/script.js"
    page = requests.get(actual_posts_url, verify=False)
    assert page.status_code != 404
  except AssertionError:
    # New Zonelets version
    actual_posts_url = Post.metadata['main_url'] + "/scripts/script.js"
    page = requests.get(actual_posts_url, verify=False)
  # Fill out metadata
  content = page.content.decode('utf-8')
  Post.metadata['authorName'] = re.findall(re.compile("let authorName = .*"), content)[0].split('"')[1]
  Post.metadata['authorName'] = re.findall(re.compile("let authorName = .*"), content)[0].split('"')[1]
  Post.metadata['authorLink'] = re.findall(re.compile("let authorLink = .*"), content)[0].split('"')[1]
  Post.metadata['blogName'] = re.findall(re.compile("let blogName = .*"), content)[0].split('"')[1]
  postsArray = re.findall(re.compile("let postsArray = .+?];", re.DOTALL), content)[0]
  posts = re.findall(r'\n\[ *"(.*?)"', postsArray)
  for post in posts:
    # Create the post
    Post(post)
  render_header = XML_HEADER.replace('_SITENAME', Post.metadata['blogName']).replace('_SITEURL', Post.metadata['starting_url']).replace('_SITETITLE', Post.metadata['authorName']).replace('_RFC822_FORMAT_DATE', TODAY_RFC822)
  print(render_header)
  for name, post in Post.posts.items():
    render_middle = XML_MIDDLE.replace('_TITLE', post.name).replace('_PAGENAME', post.relative_path).replace('_LINK', post.absolute_url).replace('_CONTENT', post.content).replace('_PUBDATE', post.rfc822date)
    print(render_middle.encode('ascii', 'ignore').decode('ascii')) # old version of Python on the server requires this
  print(MY_POST.replace('_SITEURL', Post.metadata['starting_url']))
  print(XML_FOOTER)

if __name__ == "__main__":
  main()
